<?php

defined('PHPFOX') or exit('No dice!');

/**
 * In the custom_relation_data table the values for status_id are
 * 1: has not been confirmed by `with_user_id`
 * 2: has been confirmed by `with_user_id` 
 * 3: has been denied by `with_user_id` // useful to hide it
 */
class Custom_Service_Relation_Process extends Phpfox_Service
{

	private $_sLangId;
	public function __construct()
	{
		$this->_sLangId = Phpfox::getUserBy('language_id');
		if (empty($this->_sLangId))
		{
			$aLang = Phpfox::getService('language')->get('is_default = 1');
			$this->_sLangId = $aLang[0]['language_id'];
		}
		$this->_sTable = Phpfox::getT('custom_relation');
	}

	/**
	 * This is the auxiliary function to add new relationship statuses
	 * @param String $sType can be "new", "feed_with" or "feed_new"
	 * @param Aarray $aVals array of values to get info from, passed from a form submit
	 */
	private function _add($sType, $aVals, $sStatusName = '')
	{
		if (empty($sStatusName))
		{
			$sStatusName = $aVals[$sType][$this->_sLangId];
		}

		/* Check if user entered a language phrase for the status name */
		if (preg_match('/\{phrase var=\'([a-z\._]+)\'/', $sStatusName, $aMatch))
		{
			foreach ($aVals[$sType] as $sLang => $sText)
			{
				$aVals[$sType][$sLang] = Phpfox::getPhrase($aMatch[1], array(), false, null, $sLang);
			}
		}

		/* We need the phrase var to add to this->_sTable */
		$sStatusName = Phpfox::getService('language.phrase.process')->add(array(
					'var_name' => 'custom_relation_' . $sStatusName,
					'product_id' => 'phpfox',
					'module' => 'custom|custom',
					'text' => $aVals[$sType]
				));
		$this->database()->insert($this->_sTable, array('phrase_var_name' => $sStatusName));
		return $sStatusName;
	}

	/**
	 * Auxiliary function to update relationship statuses
	 * @param String $sType can be "new", "feed_with" or "feed_new"
	 * @param Array $aVals array of values to get info from, passed from a form submit
	 * @return 
	 */
	private function _update($sType, $aVals)
	{
		/* get the phrase id */
		$aNew = array_keys($aVals[$sType]);
		$sPhraseVar = $aNew[0];
		foreach ($aVals[$sType][$aNew[0]] as $sLanguage => $sText)
		{
			$aParts = explode('.', $aNew[0]);
			$iPhraseId = $this->database()->select('phrase_id')
					->from(Phpfox::getT('language_phrase'))
					->where('module_id = "custom" AND var_name = "' . $aParts[1] . '" AND language_id = "' . $sLanguage . '"')
					->execute('getSlaveField');
			if ((int) $iPhraseId < 1)
			{
				/* phrase does not exist ? */
				die('Oops no phrase id');
				return Phpfox_Error::set('Phrase for new could not be found'); /* this should never happen */
			}
			Phpfox::getService('language.phrase.process')->update($iPhraseId, $sText);
			return ($aParts[1]);
		}
	}

	/**
	 * This function adds a new relationships status:
	 * 	- Adds an entry in custom_relation
	 * 	- Adds language phrases
	 * 
	 * It also updates existing ones:
	 * 	- if instead of a language_id a phrase is passed
	 * 	- it updates the phrase instead of creating a new one
	 * @param array $aVals 
	 * @return boolean
	 */
	public function add($aVals)
	{
		if (!isset($aVals['new']))
		{
			return Phpfox_Error::set('Invalid array');
		}		

		$sLanguageVar = '';
		if (isset($aVals['new']) && isset($aVals['new'][$this->_sLangId]))
		{
			/* This is a new phrase */
			$sLanguageVar = $this->_add('new', $aVals);
		}
		elseif (isset($aVals['new']))
		{
			/* User is updating the existing phrase */
			$sLanguageVar = $this->_update('new', $aVals);
		}

		$aParts = explode('.', $sLanguageVar);
		if (isset($aVals['feed_with']) && isset($aVals['feed_with'][$this->_sLangId]))
		{
			/* This is a new phrase but the status already exists, otherwise $aVals['feed_with'] would 
			 * have a phrase var*/
			$sStatusName = Phpfox::getService('language.phrase.process')->add(array(
					'var_name' => '' . $aParts[1] . '_with',
					'product_id' => 'phpfox',
					'module' => 'custom|custom',
					'text' => $aVals['feed_with']
				));
			//$this->_add('feed_with', $aVals, $aVals['new'][$this->_sLangId]);
		}
		elseif (isset($aVals['feed_with']))
		{
			/* User is updating the existing phrase */
			$this->_update('feed_with', $aVals);
		}

		if (isset($aVals['feed_new']) && isset($aVals['feed_new'][$this->_sLangId]))
		{
			/* This is a new phrase */
			$sStatusName = Phpfox::getService('language.phrase.process')->add(array(
					'var_name' => '' . $aParts[1] . '_new',
					'product_id' => 'phpfox',
					'module' => 'custom|custom',
					'text' => $aVals['feed_new']
				));
			//$this->_add('feed_new', $aVals, $aVals['new'][$this->_sLangId]);
		}
		elseif (isset($aVals['feed_new']))
		{
			/* User is updating the existing phrase */
			$this->_update('feed_new', $aVals);
		}
		
		return true;
	}

	/**
	 * This function deletes a relationship status and also deletes
	 * the language phrase associated with it
	 * @param type $iId
	 * @return type 
	 */
	public function delete($iId)
	{
		$aStatus = $this->database()->select('*')
				->from($this->_sTable)
				->where('relation_id = ' . (int) $iId)
				->execute('getSlaveRow');

		if (empty($aStatus))
		{
			return Phpfox_Error::set('Status id does not exist');
		}

		$this->database()->delete($this->_sTable, 'relation_id = ' . (int) $iId);
		Phpfox::getService('language.phrase.process')->delete($aStatus['phrase_var_name'], true);
		return true;
	}

	/**
	 * This function sets the relationship status for Phpfox::getUserId()
	 * by inserting into custom_relation_data
	 * status
	 * 		1 => user added the status but has not been confirmed by user_with
	 * 		2 => user and user_with confirmed the status
	 * if the user is only setting its relationship status and not defining
	 * 		another user then the status_id is set to 2 as its confirmed by
	 * 		the only user involved
	 * @param int $iRelationType Type of relation (married, engaged, single...)
	 * @param int $iWith The other other involved in the relation
	 * @param int $iUser The current user taking action
	 */
	public function updateRelationship($iRelationType, $iWith = null, $iUser = null)
	{
		if ($iUser == null)
		{
			$iUser = Phpfox::getUserId();
		}

		
		/* check if this user was listed for a relationship with another user */
		$aExisting = $this->database()->select('crd.relation_data_id, cr.phrase_var_name, u.full_name as full_name_with')
				->from(Phpfox::getT('custom_relation_data'), 'crd')
				->join(Phpfox::getT('custom_relation'), 'cr', 'cr.relation_id = crd.relation_id')
				->join(Phpfox::getT('user'), 'u', 'u.user_id = crd.user_id')
				->where('crd.with_user_id = ' . (int) $iUser . ' AND crd.user_id = ' . (int) $iWith)
				->execute('getSlaveRow');

		//die();
		/* We should have a way of knowing if this user is updating to the same 
		 * latest status, in which case he'd be just spamming the DB and feed, we can do 
		 * that by changing the default value of iWith to iUser both here and in the database
		 * no... that wouldnt work... */

		/* lets add a plug in here so if anyone wants a different behavior they can still get it */
		if ($sPlugin = Phpfox_Plugin::get('custom.service_relation_process_updaterelationship__1'))
		{
			eval($sPlugin);
			if (isset($mReturnPlugin))
			{
				return $mReturnPlugin;
			}
		}

		if (isset($aExisting['relation_data_id']) && !empty($aExisting['relation_data_id']))
		{
			/* we update because this user is confirming the relationship with the other user */
			$this->database()->update(Phpfox::getT('custom_relation_data'), array(
				'status_id' => '2'
					), 'relation_data_id = ' . $aExisting['relation_data_id']);
			/* Send an email and insert in the feed table */
			Phpfox::getLib('mail')->to(array($iUser, $iWith))
					->subject('Relationship status confirmed')
					->message(Phpfox::getUserBy('full_name') . ' and ' . $aExisting['full_name_with'] . ' are now ' . Phpfox::getPhrase($aExisting['phrase_var_name']))
					->send();
			/* Add the feed */
			// Phpfox::getService('feed.process')->add('custom_relation', $aExisting['relation_data_id'], 0, 0, $iWith, $iUser);
			Phpfox::getService('feed.process')->add('custom_relation', $aExisting['relation_data_id'], 0, 0);
			
			$this->database()->delete(Phpfox::getT('friend_request'), 'relation_data_id = ' . (int) $aExisting['relation_data_id']);

			return true;
		}


		/* Make sure this user is not adding the same type of relationship */
		$aExisting = $this->database()->select('crd.relation_data_id, crd.with_user_id, crd.relation_id')
				->from(Phpfox::getT('custom_relation_data'), 'crd')
				//->where('crd.with_user_id = ' . (int)$iWith  . ' AND crd.user_id = ' . (int) $iUser . ' AND relation_id = ' . (int)$iRelationType)
				->where('crd.user_id = ' . (int)$iUser)
				->order('relation_data_id DESC')
				->execute('getSlaveRow');
		
		if (isset($aExisting['relation_data_id']) && !empty($aExisting['relation_data_id']) 
				&& isset($aExisting['with_user_id']) && $aExisting['with_user_id'] == (int)$iWith
				&& isset($aExisting['relation_id']) && $aExisting['relation_id'] == (int)$iRelationType)
		{
			//d(__LINE__, true);
			return false;
		}
		/* User is starting a relationship */

		/* We delete all previous relationships of this same type */
		/* we use status_id = 1 here because its a new relation */
		$this->database()->delete(Phpfox::getT('custom_relation_data'), '(user_id = ' . (int) $iUser . ' OR with_user_id = ' . (int) $iUser . ') AND status_id = 1');

		/* add to the relation_data table */
		$aExisting['relation_data_id'] = $this->database()->insert(Phpfox::getT('custom_relation_data'), array(
					'relation_id' => (int) $iRelationType,
					'user_id' => (int) $iUser,
					'with_user_id' => (int) $iWith,
					'status_id' => ($iWith === null) ? 2 : 1
				));

		/* it would be zero if user is setting to single or no state */
		if (((int) $iWith) > 0)
		{
			/* Add the "notification", the matching part will be built later */
			$this->database()->insert(Phpfox::getT('friend_request'), array(
				'user_id' => (int) $iWith,
				'friend_user_id' => (int) $iUser,
				'relation_data_id' => $aExisting['relation_data_id'],
				'time_stamp' => PHPFOX_TIME
			));
		}

		if (!isset($aExisting['phrase_var_name']))
		{
			$aExisting['phrase_var_name'] = $this->database()->select('phrase_var_name')
					->from(Phpfox::getT('custom_relation'))
					->where('relation_id = ' . (int) $iRelationType)
					->execute('getSlaveField');
		}


		/* Send an email and insert in the feed table */
		if ($iWith != $iUser)
		{
			Phpfox::getLib('mail')->to(array($iUser, $iWith))
					->subject('Relationship status confirmation')
					->message(Phpfox::getUserBy('full_name') . ' wants to list you both in a relationship: ' . Phpfox::getPhrase($aExisting['phrase_var_name']))
					->send();
		}

		/* This happens only when the user is setting the status without giving 
		 * another, i.e. user only "engaged", not "engaged to..." */
		if ($iWith === null)
		{
			$iWith = $iUser;
		}
		/* Add the feed */
		// Phpfox::getService('feed.process')->add('custom_relation', $aExisting['relation_data_id'], 0, 0, $iWith, $iUser);
		Phpfox::getService('feed.process')->add('custom_relation', $aExisting['relation_data_id']);

		return true;
	}

	/**
	 * PHP version of http://bit.ly/4DdLtW
	 * This function updates the custom_relation_data table setting status_id = 3
	 * @param int $iStatusId 
	 * @param mixed $iUser if this param is available and is an integer it is 
	 * 		treated as a condition so only deny $iStatusId if $iUser is the 
	 * 		receiver of the request
	 * @return boolean
	 */
	public function _denyStatus($iStatusId, $iUser = null)
	{
		/* Make sure the status id exists (we may send a notification so this 
		 * has to exist) */
		$aRequest = Phpfox::getService('custom.relation')->getDataById($iStatusId);
		if (empty($aRequest))
		{
			return false;
		}

		if ($iUser === null ||
				(((int) $iUser) > 0 && $aRequest['with_user_id'] == $iUser))
		{
			$this->database()->update(Phpfox::getT('custom_relation_data'), array(
				'status_id' => 3), 'relation_data_id = ' . (int) $iStatusId);
			return true;
		}

		return false;
	}

	/**
	 * If a call is made to an unknown method attempt to connect
	 * it to a specific plug-in with the same name thus allowing 
	 * plug-in developers the ability to extend classes.
	 *
	 * @param string $sMethod is the name of the method
	 * @param array $aArguments is the array of arguments of being passed
	 */
	public function __call($sMethod, $aArguments)
	{
		/**
		 * Check if such a plug-in exists and if it does call it.
		 */
		if ($sPlugin = Phpfox_Plugin::get('custom.service_relation_process__call'))
		{
			return eval($sPlugin);
		}

		/**
		 * No method or plug-in found we must throw a error.
		 */
		Phpfox_Error::trigger('Call to undefined method ' . __CLASS__ . '::' . $sMethod . '()', E_USER_ERROR);
	}

}

?>